import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;

import com.mindfusion.drawing.*;
import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;


public class DecisionLayoutFrame extends JFrame
{
	private Diagram diagram;
	private JPanel pnlButtonsPanel;
	private JButton btnRandomDiagram; 
	private Brush terminalBrush;
	private Brush decisionBrush;
	private Brush normalBrush;
	
	public DecisionLayoutFrame()
	{
		super("MindFusion.Diagramming Sample: Decision Layout");
		
		terminalBrush = new GradientBrush(new float[]{0.3f, 0.7f, 0.999f},
				new Color[]{
				Color.decode("#FFFFC0"),
				Color.decode("#BFBF90"),
				Color.decode("#FFFFC0")
		}, 90);
		decisionBrush = new GradientBrush(new float[]{0.3f, 0.7f, 0.999f},
				new Color[]{
				Color.decode("#F0FFFF"),
				Color.decode("#7FDF8F"),
				Color.decode("#F0FFFF")
		}, 90);
		normalBrush = new GradientBrush(new float[]{0.3f, 0.7f, 0.999f},
				new Color[]{
				Color.decode("#E0FFFF"),
				Color.decode("#A8BFBF"),
				Color.decode("#E0FFFF")
		}, 90);
		
		this.setBounds(0, 0, 1000, 600);
		this.setVisible(true);
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		
		diagram = new Diagram();
		diagram.setBounds(new Rectangle2D.Float(0,0,1000,600));
		diagram.setMeasureUnit(GraphicsUnit.Millimeter);
		diagram.setBackBrush(new SolidBrush(Color.white));
		
		DiagramView view = new DiagramView();
		view.setDiagram(diagram);
		JScrollPane scrPane = new JScrollPane(view);
		scrPane.setVisible(true);
		
		pnlButtonsPanel = new JPanel();
		
		btnRandomDiagram = new JButton("Layout Random Diagram");
		btnRandomDiagram.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent e)
			{
				layoutRandomDiagram();
			}
			
		});
		
		pnlButtonsPanel.setLayout(new FlowLayout());
		pnlButtonsPanel.setBounds(0, 520, 200,50);
		pnlButtonsPanel.setVisible(true);
		pnlButtonsPanel.add(btnRandomDiagram);
		
		view.setDiagram(diagram);
		view.setBounds(0,0,1395, 1110);
		view.setVisible(true);
		
		this.setLayout(new BorderLayout());
		this.add(scrPane, BorderLayout.CENTER);
		this.add(pnlButtonsPanel, BorderLayout.PAGE_END);
		repaint();
	}
	
	private void layoutRandomDiagram()
	{
		diagram.clearAll();
		
		ShapeNode startNode = diagram.getFactory().createShapeNode(0, 0, 30, 10);
		startNode.setBrush(terminalBrush);
		startNode.setFont(new Font("Arial", Font.BOLD, 4));
		startNode.setText("START");
		startNode.setId("start");
		startNode.setShape(Shape.fromId("Ellipse"));
		startNode.setPen(new Pen(Color.black, 1));
		diagram.add(startNode);
		
		ShapeNode endNode = diagram.getFactory().createShapeNode(0, 0, 30, 10);
		endNode.setBrush(terminalBrush);
		endNode.setFont(new Font("Arial", Font.BOLD, 4));
		endNode.setText("END");
		endNode.setId("end");
		endNode.setShape(Shape.fromId("Ellipse"));
		endNode.setPen(new Pen(Color.black, 1));
		diagram.add(endNode);
		
		final int maxLength = 10;
		ArrayList<ShapeNode> all = new ArrayList<ShapeNode>();
		Queue<ArrayList<ShapeNode>> remaining = new LinkedList<ArrayList<ShapeNode>>();
		
		ArrayList<ShapeNode> start = new ArrayList<ShapeNode>();
		start.add(startNode);
		
		remaining.add(start);
		
		Random random = new Random();
		int length = 0;
		while (remaining.size() > 0)
		{
			ArrayList<ShapeNode> newNodes = new ArrayList<ShapeNode>();
			boolean createdNew = false;
			
			ArrayList<ShapeNode> next = remaining.remove();
			for (int i = 0; i < next.size(); i++)
			{
				ShapeNode node = next.get(i);
				boolean isDecision = node.getTag() instanceof Boolean;
				
				ShapeNode previousChoice = null;
				int outLinks = isDecision ? 2 : 1;
				String[] linkTexts = isDecision ? new String[]{"YES", "NO"} : new String[]{""};
				for (int j = 0; j < outLinks; j++)
				{
					// Create new or link to an existing node;
					// Create at least one new node for each level
					boolean createNew = random.nextInt(maxLength) > length;
					if (!createdNew && i == next.size() - 1)
						createNew = true;
					if (length == maxLength)
						createNew = false;
					
					DiagramLink link;
					if (createNew)
					{
						// New decision or an operation node
						boolean newDecision = random.nextInt(3) > 0 && !isDecision;
						ShapeNode newNode = diagram.getFactory().createShapeNode(0, 0, 30, 10);
						if (newDecision)
						{
							newNode.setShape(Shape.fromId("Decision"));
							newNode.setAnchorPattern(AnchorPattern.fromId("Decision2In2Out"));
							newNode.setBrush(decisionBrush);
							newNode.setText("Decision");
							newNode.setTag(true);
						}
						else
						{
							newNode.setShape(Shape.fromId("Rectangle"));
							newNode.setBrush(normalBrush);
							newNode.setText("Operation");
						}
						
						link = diagram.getFactory().createDiagramLink(node, newNode);
						newNodes.add(newNode);
						createdNew = true;
					}
					else
					{
						// Link to an existing node. If length == maxLength, link to the end node
						if (length == maxLength || remaining.size() == 0)
						{
							link = diagram.getFactory().createDiagramLink(node, endNode);
						}
						else
						{
							// Make sure both choices don't lead to the same node
							ShapeNode choice = null;
							while (choice == previousChoice)
							{
								choice = all.get(random.nextInt(all.size()));
							}
							link = diagram.getFactory().createDiagramLink(node, choice);
							previousChoice = choice;
						}
					}
					link.setText(linkTexts[j]);
				}
				all.add(node);
			}
			if (newNodes.size() > 0)
				remaining.add(newNodes);
			
			length++;
		}
		arrangeDiagram();
	}
	
	private void arrangeDiagram()
	{
		DecisionLayout layout = new DecisionLayout();
		layout.setHorizontalPadding(10);
		layout.setVerticalPadding(10);
		layout.setStartNode(diagram.findNodeById("start"));
		layout.arrange(diagram);
	}
	
	private static final long serialVersionUID = 1L;
}